32static const char *
const var_names[] = {
"VOLUME",
"CHANNEL",
"PEAK",
NULL };
67#define OFFSET(x) offsetof(ShowVolumeContext, x)
68#define FLAGS AV_OPT_FLAG_FILTERING_PARAM|AV_OPT_FLAG_VIDEO_PARAM
138 for (
int i = 0;
i < nb_samples;
i++)
147 for (
int i = 0;
i < nb_samples;
i++)
149 *rms =
sqrtf(sum / nb_samples);
176 if (
s->draw_persistent_duration > 0.) {
177 s->persistent_max_frames = (int)
FFMAX(
av_q2d(
s->frame_rate) *
s->draw_persistent_duration, 1.);
180 if (!
s->max_persistent ||
181 !
s->nb_frames_max_display)
194 if (
s->orientation) {
209 for (
i = 0;
i <
s->w;
i++) {
230 for (
i = 0; txt[
i];
i++) {
234 for (char_y = font_height - 1; char_y >= 0; char_y--) {
235 uint8_t *p = pic->
data[0] + (y +
i * 10) * pic->
linesize[0] + x * 4;
237 if (font[txt[
i] * font_height + font_height - 1 - char_y] &
mask)
243 uint8_t *p = pic->
data[0] + y * pic->
linesize[0] + (x +
i * 8) * 4;
244 for (char_y = 0; char_y < font_height; char_y++) {
246 if (font[txt[
i] * font_height + char_y] &
mask)
259 const uint32_t bg = (uint32_t)(
s->bgopacity * 255) << 24;
261 for (
i = 0;
i < outlink->
h;
i++) {
262 uint32_t *
dst = (uint32_t *)(
s->out->data[0] +
i *
s->out->linesize[0]);
263 for (j = 0; j < outlink->
w; j++)
271 if (
s->display_scale ==
LINEAR) {
276 if (
s->orientation) {
277 return outlink->
h - outlink->
h * max_val;
279 return s->w * max_val;
286 if ((
max >=
s->max_persistent[
channel]) || (
s->nb_frames_max_display[
channel] >=
s->persistent_max_frames)) {
288 s->nb_frames_max_display[
channel] = 0;
290 s->nb_frames_max_display[
channel] += 1;
297 if (
s->orientation) {
298 uint8_t *
dst =
s->out->data[0] + max_draw *
s->out->linesize[0] +
channel * (
s->b +
s->h) * 4;
299 for (k = 0; k <
s->h; k++) {
300 memcpy(
dst + k * 4,
s->persistant_max_rgba,
sizeof(
s->persistant_max_rgba));
303 for (k = 0; k <
s->h; k++) {
305 memcpy(
dst + max_draw * 4,
s->persistant_max_rgba,
sizeof(
s->persistant_max_rgba));
315 const int step =
s->step;
316 int c, j, k, max_draw, ret;
320 if (!
s->out ||
s->out->width != outlink->
w ||
321 s->out->height != outlink->
h) {
331 s->out->duration = 1;
333 if ((
s->f < 1.) && (
s->f > 0.)) {
334 for (j = 0; j < outlink->
h; j++) {
335 uint8_t *
dst =
s->out->data[0] + j *
s->out->linesize[0];
336 const uint32_t
alpha =
s->bgopacity * 255;
338 for (k = 0; k < outlink->
w; k++) {
345 }
else if (
s->f == 0.) {
349 if (
s->orientation) {
352 uint32_t *lut =
s->color_lut +
s->w *
c;
362 for (j =
s->w - 1; j >= max_draw; j--) {
363 uint8_t *
dst =
s->out->data[0] + j *
s->out->linesize[0] +
c * (
s->b +
s->h) * 4;
364 for (k = 0; k <
s->h; k++) {
371 if (
s->draw_persistent_duration > 0.) {
380 uint32_t *lut =
s->color_lut +
s->w *
c;
390 for (j = 0; j <
s->h; j++) {
391 uint8_t *
dst =
s->out->data[0] + (
c *
s->h +
c *
s->b + j) *
s->out->linesize[0];
393 for (k = 0; k < max_draw; k++) {
400 if (
s->draw_persistent_duration > 0.) {
420 if (
s->orientation) {
437 if (
s->orientation) {
506 .p.name =
"showvolume",
508 .p.priv_class = &showvolume_class,
uint8_t ptrdiff_t const uint8_t ptrdiff_t int intptr_t intptr_t int int16_t * dst
static enum AVSampleFormat sample_fmts[]
static int query_formats(const AVFilterContext *ctx, AVFilterFormatsConfig **cfg_in, AVFilterFormatsConfig **cfg_out)
static int config_input(AVFilterLink *inlink)
const FFFilter ff_avf_showvolume
static const AVFilterPad showvolume_outputs[]
static int filter_frame(AVFilterLink *inlink, AVFrame *insamples)
static void draw_max_line(ShowVolumeContext *s, int max_draw, int channel)
static void find_rms(float *src, int nb_samples, float *rms)
static const AVFilterPad showvolume_inputs[]
static int calc_max_draw(ShowVolumeContext *s, AVFilterLink *outlink, float max)
static const AVOption showvolume_options[]
static int config_input(AVFilterLink *inlink)
static void clear_picture(ShowVolumeContext *s, AVFilterLink *outlink)
static void drawtext(AVFrame *pic, int x, int y, const char *txt, int o)
static int query_formats(const AVFilterContext *ctx, AVFilterFormatsConfig **cfg_in, AVFilterFormatsConfig **cfg_out)
static int activate(AVFilterContext *ctx)
static av_cold void uninit(AVFilterContext *ctx)
static void calc_persistent_max(ShowVolumeContext *s, float max, int channel)
static int config_output(AVFilterLink *outlink)
static void find_peak(float *src, int nb_samples, float *peak)
int ff_filter_frame(AVFilterLink *link, AVFrame *frame)
Send a frame of data to the next filter.
int ff_inlink_consume_samples(AVFilterLink *link, unsigned min, unsigned max, AVFrame **rframe)
Take samples from the link's FIFO and update the link's stats.
void ff_filter_set_ready(AVFilterContext *filter, unsigned priority)
Mark a filter ready and schedule it for activation.
int ff_inlink_make_frame_writable(AVFilterLink *link, AVFrame **rframe)
Make sure a frame is writable.
int ff_inlink_queued_samples(AVFilterLink *link)
Main libavfilter public API header.
#define i(width, name, range_min, range_max)
Public libavutil channel layout APIs header.
static __device__ float sqrtf(float a)
static __device__ float fabsf(float a)
static int32_t find_peak(DCAEncContext *c, const int32_t *in, int len)
float fmaxf(float, float)
int(* init)(AVBSFContext *ctx)
channel
Use these values when setting the channel map with ebur128_set_channel().
void av_expr_free(AVExpr *e)
Free a parsed expression previously created with av_expr_parse().
double av_expr_eval(AVExpr *e, const double *const_values, void *opaque)
Evaluate a previously parsed expression.
int av_expr_parse(AVExpr **expr, const char *s, const char *const *const_names, const char *const *func1_names, double(*const *funcs1)(void *, double), const char *const *func2_names, double(*const *funcs2)(void *, double, double), int log_offset, void *log_ctx)
Parse an expression.
simple arithmetic expression evaluator
@ AV_OPT_TYPE_CONST
Special option type for declaring named constants.
@ AV_OPT_TYPE_VIDEO_RATE
Underlying C type is AVRational.
@ AV_OPT_TYPE_INT
Underlying C type is int.
@ AV_OPT_TYPE_DOUBLE
Underlying C type is double.
@ AV_OPT_TYPE_FLOAT
Underlying C type is float.
@ AV_OPT_TYPE_BOOL
Underlying C type is int.
@ AV_OPT_TYPE_STRING
Underlying C type is a uint8_t* that is either NULL or points to a C string allocated with the av_mal...
@ AV_OPT_TYPE_COLOR
Underlying C type is uint8_t[4].
int av_channel_name(char *buf, size_t buf_size, enum AVChannel channel_id)
Get a human readable string in an abbreviated form describing a given channel.
enum AVChannel av_channel_layout_channel_from_index(const AVChannelLayout *channel_layout, unsigned int idx)
Get the channel with the given index in a channel layout.
#define AVERROR_BUG
Internal bug, also see AVERROR_BUG2.
void av_frame_free(AVFrame **frame)
Free the frame and any dynamically allocated objects in it, e.g.
AVFrame * av_frame_clone(const AVFrame *src)
Create a new frame that references the same data as src.
static double av_q2d(AVRational a)
Convert an AVRational to a double.
static av_always_inline AVRational av_inv_q(AVRational q)
Invert a rational.
int64_t av_rescale(int64_t a, int64_t b, int64_t c)
Rescale a 64-bit integer with rounding to nearest.
int64_t av_rescale_q(int64_t a, AVRational bq, AVRational cq)
Rescale a 64-bit integer by 2 rational numbers.
AVSampleFormat
Audio sample formats.
@ AV_SAMPLE_FMT_FLTP
float, planar
static const int16_t alpha[]
static av_cold void uninit(AVBitStreamFilterContext *ctx)
static int activate(AVBitStreamFilterContext *ctx)
static int config_output(AVBitStreamFilterLink *outlink)
#define FILTER_INPUTS(array)
#define FILTER_OUTPUTS(array)
#define FF_FILTER_FORWARD_WANTED(outlink, inlink)
Forward the frame_wanted_out flag from an output link to an input link.
#define FF_FILTER_FORWARD_STATUS(inlink, outlink)
Acknowledge the status on an input link and forward it to an output link.
#define FFERROR_NOT_READY
Filters implementation helper functions and internal structures.
#define FF_FILTER_FORWARD_STATUS_BACK(outlink, inlink)
Forward the status on an output link to an input link.
static FilterLink * ff_filter_link(AVFilterLink *link)
#define AVFILTER_DEFINE_CLASS(fname)
#define FILTER_QUERY_FUNC2(func)
#define NULL_IF_CONFIG_SMALL(x)
Return NULL if CONFIG_SMALL is true, otherwise the argument without modification.
static enum AVPixelFormat pix_fmts[]
static const uint16_t mask[17]
void * av_calloc(size_t nmemb, size_t size)
Memory handling functions.
static const char *const var_names[]
AVPixelFormat
Pixel format.
@ AV_PIX_FMT_RGBA
packed RGBA 8:8:8:8, 32bpp, RGBARGBA...
static void draw_text(FFDrawContext *draw, AVFrame *out, FFDrawColor *color, int x0, int y0, const uint8_t *text)
int nb_channels
Number of channels in this layout.
Describe the class of an AVClass context structure.
AVFilterLink ** inputs
array of pointers to input links
void * priv
private data for use by the filter
A link between two filters.
int w
agreed upon image width
int h
agreed upon image height
AVFilterContext * src
source filter
AVRational time_base
Define the time base used by the PTS of the frames/samples which will pass through this link.
int sample_rate
samples per second
AVChannelLayout ch_layout
channel layout of current buffer (see libavutil/channel_layout.h)
AVRational sample_aspect_ratio
agreed upon sample aspect ratio
AVFilterContext * dst
dest filter
A filter pad used for either input or output.
This structure describes decoded (raw) audio or video data.
int nb_samples
number of audio samples (per channel) described by this frame
int64_t pts
Presentation timestamp in time_base units (time when frame should be shown to user).
uint8_t * data[AV_NUM_DATA_POINTERS]
pointer to the picture/channel planes.
int linesize[AV_NUM_DATA_POINTERS]
For video, a positive or negative value, which is typically indicating the size in bytes of each pict...
uint8_t ** extended_data
pointers to the data planes/channels.
Rational number (pair of numerator and denominator).
Link properties exposed to filter code, but not external callers.
AVRational frame_rate
Frame rate of the stream on the link, or 1/0 if unknown or variable.
int * nb_frames_max_display
int persistent_max_frames
double draw_persistent_duration
void(* meter)(float *src, int nb_samples, float *max)
uint8_t persistant_max_rgba[4]
static AVFormatContext * ctx
AVFrame * ff_get_video_buffer(AVFilterLink *link, int w, int h)
Request a picture buffer with a specific set of permissions.
const uint8_t * avpriv_cga_font_get(void)
CGA/EGA/VGA ROM font data.